@@ -21,89 +21,89 @@ S3_DEFAULT_REGION = "us-west-1"
 S3_DEFAULT_PROFILE = "default"
 #S3_WARNING_GIVEN = "0"
 
-def create_session_key(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    if (region is None and profile is None):
+def create_session_key(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    if (s3_region is None and aws_profile is None):
         return "DEFAULT_KEY"
     else:
-        return region + ":" + profile
+        return s3_region + ":" + aws_profile
 
-def get_s3_session(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
+def get_s3_session(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
 
     # generate s3_session
-    if (region is not None and profile is not None):
-        utils.debug("get_s3_session: region: {}, profile: {}".format(region, profile))
-        session = boto3.session.Session(region_name = region, profile_name = profile)
+    if (s3_region is not None and aws_profile is not None):
+        utils.debug("get_s3_session: s3_region: {}, aws_profile: {}".format(s3_region, aws_profile))
+        session = boto3.session.Session(region_name = s3_region, profile_name = aws_profile)
     else:
-        utils.debug("get_s3_session: no region or profile")
+        utils.debug("get_s3_session: no s3_region or aws_profile")
         session = boto3.session.Session()
 
     # return
     return session
 
-def get_s3_session_cache(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    key = create_session_key(region, profile)
+def get_s3_session_cache(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    key = create_session_key(s3_region, aws_profile)
 
     # make it thread safe
     with S3_SESSION_LOCK:
         if ((key in S3_SESSIONS.keys()) == False):
-            S3_SESSIONS[key] = get_s3_session(region, profile)
+            S3_SESSIONS[key] = get_s3_session(s3_region, aws_profile)
 
     return S3_SESSIONS[key]
 
 # TODO
-def get_s3_resource(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    session = get_s3_session_cache(region, profile)
+def get_s3_resource(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    session = get_s3_session_cache(s3_region, aws_profile)
 
     return session.resource("s3")
 
-def get_s3_resource_cache(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    key = create_session_key(region, profile)
+def get_s3_resource_cache(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    key = create_session_key(s3_region, aws_profile)
 
     # make it thread safe
     with S3_RESOURCE_LOCK:
         if ((key in S3_RESOURCE.keys()) == False):
-            S3_RESOURCE[key] = get_s3_resource(region, profile)
+            S3_RESOURCE[key] = get_s3_resource(s3_region, aws_profile)
 
     return S3_RESOURCE[key]
 
-def get_s3_client(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    session = get_s3_session_cache(region, profile)
+def get_s3_client(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    session = get_s3_session_cache(s3_region, aws_profile)
 
     return session.client("s3")
 
-def get_s3_client_cache(region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    key = create_session_key(region, profile)
+def get_s3_client_cache(s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    key = create_session_key(s3_region, aws_profile)
 
     # make it thread safe
     with S3_CLIENT_LOCK:
         if ((key in S3_CLIENTS.keys()) == False):
-            S3_CLIENTS[key] = get_s3_client(region, profile)
+            S3_CLIENTS[key] = get_s3_client(s3_region, aws_profile)
 
     return S3_CLIENTS[key]
 
-def get_s3_bucket(bucket_name, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_resource_cache(region, profile)
+def get_s3_bucket(bucket_name, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_resource_cache(s3_region, aws_profile)
 
     return s3.Bucket(bucket_name)
 
-def get_s3_bucket_cache(bucket_name, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
+def get_s3_bucket_cache(bucket_name, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
     if ((bucket_name in S3_BUCKETS.keys()) == False):
-        S3_BUCKETS[bucket_name] = get_s3_bucket(bucket_name, region, profile)
+        S3_BUCKETS[bucket_name] = get_s3_bucket(bucket_name, s3_region, aws_profile)
 
     return S3_BUCKETS[bucket_name]
 
-def get_file_content(bucket_name, object_key, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3_bucket = get_s3_bucket_cache(bucket_name, region, profile)
+def get_file_content(bucket_name, object_key, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3_bucket = get_s3_bucket_cache(bucket_name, s3_region, aws_profile)
     obj = s3_bucket.Object(object_key)
     obj_content = obj.get()
     body = obj_content["Body"]
@@ -113,18 +113,18 @@ def get_file_content(bucket_name, object_key, region = None, profile = None):
     return data
 
 # TODO: Deprecated
-def get_s3_file_content(bucket_name, object_key, region = None, profile = None):
+def get_s3_file_content(bucket_name, object_key, s3_region = None, aws_profile = None):
     utils.warn_once("use get_file_content instead")
-    return get_file_content(bucket_name, object_key, region = region, profile = profile)
+    return get_file_content(bucket_name, object_key, s3_region = s3_region, aws_profile = aws_profile)
 
 # TODO: Deprecated
-def get_s3_file_content_as_text(bucket_name, object_key, region = None, profile = None):
+def get_s3_file_content_as_text(bucket_name, object_key, s3_region = None, aws_profile = None):
     utils.warn_once("use get_file_content_as_text instead")
-    return get_file_content_as_text(bucket_name, object_key, region = region, profile = profile)
+    return get_file_content_as_text(bucket_name, object_key, s3_region = s3_region, aws_profile = aws_profile)
 
-def get_file_content_as_text(bucket_name, object_key, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    barr = get_file_content(bucket_name, object_key, region, profile)
+def get_file_content_as_text(bucket_name, object_key, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    barr = get_file_content(bucket_name, object_key, s3_region, aws_profile)
     if (object_key.endswith(".gz")):
         barr = gzip.decompress(barr)
     elif(object_key.endswith(".zip")):
@@ -137,17 +137,17 @@ def get_file_content_as_text(bucket_name, object_key, region = None, profile = N
     return barr.decode().rstrip("\n")
 
 # TODO: this is expensive and works in specific scenarios only especially for files
-def check_path_exists(path, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_client_cache(region, profile)
+def check_path_exists(path, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_client_cache(s3_region = s3_region, aws_profile = aws_profile)
     bucket_name, object_key = utils.split_s3_path(path)
 
     results = s3.list_objects(Bucket = bucket_name, Prefix = object_key)
     return "Contents" in results
 
-def check_file_exists(path, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_client_cache(region, profile)
+def check_file_exists(path, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_client_cache(s3_region = s3_region, aws_profile = aws_profile)
     bucket_name, object_key = utils.split_s3_path(path)
     try:
         # call head_object which is supposed to be efficient. This works only for files and not directories?
@@ -161,19 +161,19 @@ def check_file_exists(path, region = None, profile = None):
     except:
         return False
 
-def put_file_content(bucket_name, object_key, barr, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3_bucket = get_s3_resource_cache(region, profile)
+def put_file_content(bucket_name, object_key, barr, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3_bucket = get_s3_resource_cache(s3_region, aws_profile)
     obj = s3_bucket.Object(bucket_name, object_key)
     obj.put(Body = barr)
 
 # TODO: Deprecated
-def put_s3_file_content(bucket_name, object_key, barr, region = None, profile = None):
+def put_s3_file_content(bucket_name, object_key, barr, s3_region = None, aws_profile = None):
     utils.warn_once("use put_file_content instead")
-    return put_file_content(bucket_name, object_key, barr, region = None, profile = None)
+    return put_file_content(bucket_name, object_key, barr, s3_region = None, aws_profile = None)
 
-def put_file_with_text_content(bucket_name, object_key, text, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
+def put_file_with_text_content(bucket_name, object_key, text, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
     barr = str.encode(text)
     if (object_key.endswith(".gz")):
         barr = gzip.compress(barr)
@@ -185,24 +185,24 @@ def put_file_with_text_content(bucket_name, object_key, text, region = None, pro
             zfile.writestr(object_file_name[0:-4], barr)
         barr = mzip.getvalue()
 
-    put_file_content(bucket_name, object_key, barr, region = region, profile = profile)
+    put_file_content(bucket_name, object_key, barr, s3_region = s3_region, aws_profile = aws_profile)
 
 # TODO: Deprecated
-def put_s3_file_with_text_content(bucket_name, object_key, text, region = None, profile = None):
+def put_s3_file_with_text_content(bucket_name, object_key, text, s3_region = None, aws_profile = None):
     utils.warn_once("use put_file_with_text_content instead")
-    put_file_with_text_content(bucket_name, object_key, text, region = region, profile = profile)
+    put_file_with_text_content(bucket_name, object_key, text, s3_region = s3_region, aws_profile = aws_profile)
  
-def resolve_region_profile(region = None, profile = None):
-    # resolve region
-    if ((region == "" or region is None) and "S3_REGION" in os.environ.keys()):
-        region = os.environ["S3_REGION"]
+def resolve_region_profile(s3_region = None, aws_profile = None):
+    # resolve s3_region
+    if ((s3_region == "" or s3_region is None) and "S3_REGION" in os.environ.keys()):
+        s3_region = os.environ["S3_REGION"]
 
-    # resolve profile
-    if ((profile == "" or profile is None) and "AWS_PROFILE" in os.environ.keys()):
-        profile = os.environ["AWS_PROFILE"]
+    # resolve aws_profile
+    if ((aws_profile == "" or aws_profile is None) and "AWS_PROFILE" in os.environ.keys()):
+        aws_profile = os.environ["AWS_PROFILE"]
 
     # return
-    return region, profile
+    return s3_region, aws_profile
 
 # https://stackoverflow.com/questions/54314563/how-to-get-more-than-1000-objects-from-s3-by-using-list-objects-v2
 def __get_all_s3_objects__(s3, **base_kwargs):
@@ -218,21 +218,21 @@ def __get_all_s3_objects__(s3, **base_kwargs):
         continuation_token = response.get('NextContinuationToken')
 
 # FIXME: This method is implemented using reverse engineering. Not so reliable
-def get_directory_listing(path, filter_func = None, fail_if_missing = True, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_client_cache(region, profile)
+# TODO: ignore_if_missing should be FALSE by default
+def get_directory_listing(path, filter_func = None, ignore_if_missing = False, skip_exist_check = False, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_client_cache(s3_region = s3_region, aws_profile = aws_profile)
 
     # split the path
     bucket_name, object_key = utils.split_s3_path(path)
 
     filenames = []
     # validation
-    if (check_path_exists(path, region, profile) == False):
-        if (fail_if_missing):
+    if (check_path_exists(path, s3_region = s3_region, aws_profile = aws_profile) == False):
+        if (ignore_if_missing == False):
             raise Exception("Directory does not exist: " + path)
         else:
-            if (utils.is_debug()):
-                print("Directory does not exist: " + path)
+            utils.debug("Directory does not exist: {}".format(path))
             return []
 
     # set the max keys as boto3 api are weird
@@ -278,7 +278,8 @@ def get_directory_listing(path, filter_func = None, fail_if_missing = True, regi
     filenames = sorted(list(set(filenames)))
 
     # valid files as boto does a prefix search
-    filenames = list(filter(lambda t: check_path_exists(t), filenames))
+    if (skip_exist_check == False):
+        filenames = list(filter(lambda t: check_path_exists(t, s3_region = s3_region, aws_profile = aws_profile), filenames))
 
     # apply filter func if any
     if (filter_func is not None):
@@ -287,16 +288,16 @@ def get_directory_listing(path, filter_func = None, fail_if_missing = True, regi
     # return
     return filenames
 
-def delete_file(path, fail_if_missing = False, region = None, profile = None):
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_client_cache(region, profile)
+def delete_file(path, ignore_if_missing = False, s3_region = None, aws_profile = None):
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_client_cache(s3_region = s3_region, aws_profile = aws_profile)
 
     # split the path
     bucket_name, object_key = utils.split_s3_path(path)
 
     # check if the file exists
-    if (check_path_exists(path) == False):
-        if (fail_if_missing):
+    if (check_path_exists(path, s3_region = s3_region, aws_profile = aws_profile) == False):
+        if (ignore_if_missing == False):
             raise Exception("delete_file: path doesnt exist: {}".format(path))
         else:
             utils.debug("delete_file: path doesnt exist: {}".format(path))
@@ -306,10 +307,10 @@ def delete_file(path, fail_if_missing = False, region = None, profile = None):
     # delete
     s3.delete_object(Bucket = bucket_name, Key = object_key)
 
-def get_last_modified_time(path, fail_if_missing = False, region = None, profile = None):
+def get_last_modified_time(path, ignore_if_missing = False, s3_region = None, aws_profile = None):
     # check if exists
-    if (check_path_exists(path) == False):
-        if (fail_if_missing):
+    if (check_path_exists(path, s3_region = s3_region, aws_profile = aws_profile) == False):
+        if (ignore_if_missing == False):
             raise Exception("get_last_modified_time: path doesnt exist: {}".format(path))
         else:
             utils.debug("get_last_modified_time: path doesnt exist: {}".format(path))
@@ -317,8 +318,8 @@ def get_last_modified_time(path, fail_if_missing = False, region = None, profile
         return None
 
     # parse
-    region, profile = resolve_region_profile(region, profile)
-    s3 = get_s3_client_cache(region, profile)
+    s3_region, aws_profile = resolve_region_profile(s3_region, aws_profile)
+    s3 = get_s3_client_cache(s3_region = s3_region, aws_profile = aws_profile)
 
     # split the path
     bucket_name, object_key = utils.split_s3_path(path)
